"""The mix-in implementation for the text's CSS underline setting.
"""

from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.boolean import Boolean
from apysc._validation import arg_validation_decos


class TextDecorationUnderlineCssMixIn(
    AttrLinkingMixIn,
):
    _underline: Boolean

    def _initialize_underline(self) -> None:
        """
        Initialize the `_underline` attribute.
        """
        if hasattr(self, "_underline"):
            return
        self._underline = Boolean(False)

    @property
    def underline(self) -> Boolean:
        """
        Get a text underline (`text-decoration: underline`) setting.

        Returns
        -------
        underline : Boolean
            A text underline setting.

        References
        ----------
        - Text underline property
            - https://simon-ritchie.github.io/apysc/en/text_underline.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=195,
        ...     stage_elem_id="stage",
        ... )

        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=250,
        ...     fill_color=ap.Colors.GRAY_AAAAAA,
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.underline = ap.True_
        >>> multi_line_text.underline
        Boolean(True)
        """
        self._initialize_underline()
        return self._underline.copy()

    @underline.setter
    @arg_validation_decos.is_apysc_boolean(arg_position_index=1)
    def underline(self, value: Boolean) -> None:
        """
        Set a text underline (`text-decoration: underline`) setting.

        Parameters
        ----------
        value : Boolean
            A text underline setting.

        References
        ----------
        - Text underline property
            - https://simon-ritchie.github.io/apysc/en/text_underline.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=300,
        ...     stage_height=195,
        ...     stage_elem_id="stage",
        ... )

        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=250,
        ...     fill_color=ap.Colors.GRAY_AAAAAA,
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.underline = ap.True_
        >>> multi_line_text.underline
        Boolean(True)
        """
        from apysc._branch._else import Else
        from apysc._branch._if import If
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        self._underline = value

        with If(self._underline):
            interface.set_css(name="text-decoration", value="underline")
        with Else():
            interface.set_css(name="text-decoration", value="none")

        self._append_applying_new_attr_val_exp(
            new_attr=self._underline, attr_name="underline"
        )
        self._append_attr_to_linking_stack(attr=self._underline, attr_name="underline")



"""This module is for the translation mapping data of the
following document:

Document file: svg_mask.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# SvgMask class and its related interfaces": "# SvgMask クラスと関連インターフェース",
    ##################################################
    "This page explains the `SvgMask` class and related interfaces, such as the `add_svg_masking_object` method and `svg_mask` property.": "このページでは`SvgMask`クラスとそれに関連した`add_svg_masking_object`メソッドや`svg_mask`属性などのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `SvgMask` handles SVG graphics mask settings.": "`SvgMask`クラスはSVGグラフィックのマスク設定を扱います。",  # noqa
    ##################################################
    "You can set another SVG `DisplayObject` as a mask for an SVG `DisplayObject` (e.g., `Rectangle`) to display only the overlapping area.": "重なりあった領域のみを表示する形でSVGの`DisplayObject`（例 : `Rectangle`）に別のSVGの`DisplayObject`を設定することができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can apply the mask setting in the following steps:": "以下のステップでマスク設定を適用することができます。",  # noqa
    ##################################################
    "1. Create an `SvgMask` instance.": "1. `SvgMask`インスタンスを作成します。",
    ##################################################
    "2. Add a `DisplayObject` to the created `SvgMask` instance with the `add_svg_masking_object` method.": "2. `add_svg_masking_object`メソッドを使って作成した`SvgMask`のインスタンスに`DisplayObject`を追加します。",  # noqa
    ##################################################
    "3. Set a mask instance to the target `DisplayObject`'s `svg_mask` property.": "3. マスクのインスタンスを対象の`DisplayObject`の`svg_mask`属性に設定します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\n# 1. Create an `SvgMask` instance.\nmask: ap.SvgMask = ap.SvgMask()\ncircle: ap.Circle = ap.Circle(x=100, y=100, radius=50)\n\n# 2. Add a `DisplayObject` to the created `SvgMask` instance.\nmask.add_svg_masking_object(masking_object=circle)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n)\n\n# 3. Set a mask instance to the target `DisplayObject`\'s `svg_mask` property.\nrectangle.svg_mask = mask\n\nap.save_overall_html(dest_dir_path="svg_mask_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\n# 1. Create an `SvgMask` instance.\nmask: ap.SvgMask = ap.SvgMask()\ncircle: ap.Circle = ap.Circle(x=100, y=100, radius=50)\n\n# 2. Add a `DisplayObject` to the created `SvgMask` instance.\nmask.add_svg_masking_object(masking_object=circle)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n)\n\n# 3. Set a mask instance to the target `DisplayObject`\'s `svg_mask` property.\nrectangle.svg_mask = mask\n\nap.save_overall_html(dest_dir_path="svg_mask_basic_usage/")\n```',  # noqa
    ##################################################
    "## Case when you want to synchronize the coordinates of DisplayObject and mask": "## DisplayObjectとマスクの座標を同期させたい場合のケース",  # noqa
    ##################################################
    "Both the `DisplayObject` to set the mask and the `DisplayObject` for the mask have separate coordinates.": "マスクを設定する対象の`DisplayObject`とマスク用の`DisplayObject`はそれぞれ分離された座標値を持っています。",  # noqa
    ##################################################
    "If you want to change the coordinates of each `DisplayObject` by the same amount, it is convenient to use the `Sprite` container.": "もし両方の`DisplayObject`の座標を同じ量だけ変更したい場合には`Sprite`のコンテナーを使用すると便利です。",  # noqa
    ##################################################
    "By changing only the coordinates of a `Sprite` container, you can change the coordinates of a `DisplayObject` while maintaining the coordinates of the mask.": "`Sprite`のコンテナーの座標のみを変更することで、マスクの座標を維持したまま`DisplayObject`の座標を変更することができます。",  # noqa
    ##################################################
    "Notes: You do not need to add `DisplayObject` to the `Sprite` container for the masking.": "特記事項 : マスク処理用の`DisplayObject`は`Sprite`のコンテナーへと追加する必要はありません。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=400,\n    stage_height=300,\n    stage_elem_id="stage",\n)\nmask: ap.SvgMask = ap.SvgMask()\ncircle: ap.Circle = ap.Circle(x=150, y=100, radius=100)\nmask.add_svg_masking_object(masking_object=circle)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50, y=50, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n)\nrectangle.svg_mask = mask\n\nsprite: ap.Sprite = ap.Sprite()\n# Notes: You do not need to add the circle for masking.\nsprite.add_child(rectangle)\nsprite.x = ap.Number(100)\nsprite.y = ap.Number(50)\n\nap.save_overall_html(dest_dir_path="svg_mask_sprite_container_example/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=400,\n    stage_height=300,\n    stage_elem_id="stage",\n)\nmask: ap.SvgMask = ap.SvgMask()\ncircle: ap.Circle = ap.Circle(x=150, y=100, radius=100)\nmask.add_svg_masking_object(masking_object=circle)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50, y=50, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n)\nrectangle.svg_mask = mask\n\nsprite: ap.Sprite = ap.Sprite()\n# Notes: You do not need to add the circle for masking.\nsprite.add_child(rectangle)\nsprite.x = ap.Number(100)\nsprite.y = ap.Number(50)\n\nap.save_overall_html(dest_dir_path="svg_mask_sprite_container_example/")\n```',  # noqa
    ##################################################
    "## SvgMask constructor API": "## SvgMask クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The class for the SVG masking.<hr>": "SVGのマスク処理のためのクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n```",  # noqa
    ##################################################
    "## SvgMask add_svg_masking_object method API": "## SvgMask クラスの add_svg_masking_object メソッドのAPI",  # noqa
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add an SVG masking object to this mask. This instance uses its masking object to mask other SVG graphics objects. It is possible to add multiple masking objects to a mask.<hr>": "このマスクにマスク処理用のSVGのオブジェクトを追加します。このインスタンスは他のSVGのグラフィックスオブジェクトをマスクするためにそのオブジェクトを使用します。マスクへ複数のマスク処理用のオブジェクトを追加することができます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `masking_object`: GraphicsBase": "- `masking_object`: GraphicsBase",
    ##################################################
    "  - The masking object to add.": "  - 追加するマスク処理用のオブジェクト。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n```",  # noqa
    ##################################################
    "## svg_mask property API": "## svg_mask 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an SVG mask setting. If the mask is not set, this property becomes `None`.<hr>": "SVGのマスク設定を取得します。もしマスク設定がされていなければ、この属性の値はNoneとなります。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `mask`: Optional[SvgMask]": "- `mask`: Optional[SvgMask]",
    ##################################################
    "  - A mask setting.": "  - マスク設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n>>> assert rectangle.svg_mask == mask\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> mask: ap.SvgMask = ap.SvgMask()\n>>> circle: ap.Circle = ap.Circle(\n...     x=50, y=50, radius=50, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> mask.add_svg_masking_object(masking_object=circle)\n>>> rectangle: ap.Rectangle = ap.Rectangle(\n...     x=0, y=0, width=100, height=100, fill_color=ap.Colors.CYAN_00AAFF\n... )\n>>> rectangle.svg_mask = mask\n>>> assert rectangle.svg_mask == mask\n```",  # noqa
}



"""Class implementations for the left and right datetimes-related mix-in.
"""

from datetime import datetime
from typing import TYPE_CHECKING
from typing import Tuple

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting

if TYPE_CHECKING:
    from apysc._time.datetime_ import DateTime


class LeftAndRightDatetimesMixIn:
    @add_debug_info_setting(module_name=__name__)
    @final
    def _get_left_and_right_py_datetimes_from_apysc_datetime(
        self,
        *,
        left_apysc_datetime: "DateTime",
        right_apysc_datetime: "DateTime",
    ) -> Tuple[datetime, datetime]:
        """
        Get Python's left and right datetimes from specified apysc's
        `DateTime` instances.

        Parameters
        ----------
        left_apysc_datetime : DateTime
            Left-side apysc's `DateTime` instance.
        right_apysc_datetime : DateTime
            Right-side apysc's `DateTime` instance.

        Returns
        -------
        left_py_datetime : datetime
            Left-side Python's datetime instance.
        right_py_datetime : datetime
            Right-side Python's datetime instance.
        """
        left_py_datetime: datetime = datetime(
            left_apysc_datetime._year._value,
            left_apysc_datetime._month._value,
            left_apysc_datetime._day._value,
            left_apysc_datetime._hour._value,
            left_apysc_datetime._minute._value,
            left_apysc_datetime._second._value,
            left_apysc_datetime._millisecond._value * 1000,
        )
        right_py_datetime: datetime = datetime(
            right_apysc_datetime._year._value,
            right_apysc_datetime._month._value,
            right_apysc_datetime._day._value,
            right_apysc_datetime._hour._value,
            right_apysc_datetime._minute._value,
            right_apysc_datetime._second._value,
            right_apysc_datetime._millisecond._value * 1000,
        )
        return left_py_datetime, right_py_datetime



"""The class implementation for the `assignment_turned_in` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAssignmentTurnedInIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `assignment_turned_in` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M19 3h-4.18C14.4 1.84 13.3 1 12 1c-1.3 0-2.4.84-2.82 2H5c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h14c1.1 0 2-.9 2-2V5c0-1.1-.9-2-2-2zm-7 0c.55 0 1 .45 1 1s-.45 1-1 1-1-.45-1-1 .45-1 1-1zm-2 14l-4-4 1.41-1.41L10 14.17l6.59-6.59L18 9l-8 8z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: string_split.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# String class split interface": "# String クラスの split インターフェイス",
    ##################################################
    "This page explains the `String` class `split` method interface.": "このページでは`String`クラスの`split`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `split` method interface splits a string into an `Array` instance of `String`s.": "`split`メソッドのインターフェイスは文字列を分割して`String`の値を格納した`Array`の配列を作成します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `split` method requires the `sep` `String` argument as a separator.": "`split`メソッドは区切り文字としての`sep`の`String`型の引数を必要とします。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstr_value: ap.String = ap.String("Lorem ipsum dolor sit")\nsplitted_strs: ap.Array[ap.String] = str_value.split(sep=ap.String(" "))\nap.assert_arrays_equal(splitted_strs, ["Lorem", "ipsum", "dolor", "sit"])\n\nap.save_overall_html(dest_dir_path="string_split_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\nstr_value: ap.String = ap.String("Lorem ipsum dolor sit")\nsplitted_strs: ap.Array[ap.String] = str_value.split(sep=ap.String(" "))\nap.assert_arrays_equal(splitted_strs, ["Lorem", "ipsum", "dolor", "sit"])\n\nap.save_overall_html(dest_dir_path="string_split_basic_usage/")\n```',  # noqa
    ##################################################
    "## split API": "## split API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Split a current string with a separator string.<hr>": "現在の文字列を指定された区切り文字（列）を使って分割します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `sep`: String": "- `sep`: String",
    ##################################################
    "  - A separator string.": "  - 区切り文字（列）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `splitted_strs`: Array[String]": "- `splitted_strs`: Array[String]",
    ##################################################
    "  - A splitted strings' array.": "  - 分割された文字列を格納した配列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> str_value: ap.String = ap.String("Lorem ipsum dolor sit")\n>>> splitted_strs: ap.Array[ap.String] = str_value.split(sep=ap.String(" "))\n>>> ap.assert_arrays_equal(splitted_strs, ["Lorem", "ipsum", "dolor", "sit"])\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> str_value: ap.String = ap.String("Lorem ipsum dolor sit")\n>>> splitted_strs: ap.Array[ap.String] = str_value.split(sep=ap.String(" "))\n>>> ap.assert_arrays_equal(splitted_strs, ["Lorem", "ipsum", "dolor", "sit"])\n```',  # noqa
}



"""The mix-in class implementation for the `_add_border` method.
"""

from typing_extensions import final

from apysc._color.color import Color
from apysc._display.rectangle import Rectangle
from apysc._display.sprite import Sprite
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number


class AddBorderMixIn:
    _border: Rectangle

    @final
    @add_debug_info_setting(module_name=__name__)
    def _add_border(
        self,
        *,
        border_container: Sprite,
        width: Int,
        height: Int,
        border_color: Color,
        border_alpha: Number,
        border_thickness: Int,
        variable_name_suffix: str,
    ) -> None:
        """
        Add a border to a container.

        Parameters
        ----------
        border_container : Sprite
            A border container instance.
        width : Int
            A border width.
        height : Int
            A border height.
        border_color : Color
            A border color.
        border_alpha : Number
            A border alpha.
        border_thickness : Int
            A border thickness (line width).
        variable_name_suffix : str
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._border = Rectangle(
            x=0,
            y=0,
            width=width,
            height=height,
            line_color=border_color,
            line_alpha=border_alpha,
            line_thickness=border_thickness,
            variable_name_suffix=variable_name_suffix,
        )
        border_container.add_child(self._border)



"""The class implementation for the outlined `contact_page` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialContactPageOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `contact_page` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M13.17,4L18,8.83V20H6V4H13.17 M14,2H6C4.9,2,4,2.9,4,4v16c0,1.1,0.9,2,2,2h12c1.1,0,2-0.9,2-2V8L14,2L14,2z M12,14 c1.1,0,2-0.9,2-2c0-1.1-0.9-2-2-2s-2,0.9-2,2C10,13.1,10.9,14,12,14z M16,17.43c0-0.81-0.48-1.53-1.22-1.85 C13.93,15.21,12.99,15,12,15c-0.99,0-1.93,0.21-2.78,0.58C8.48,15.9,8,16.62,8,17.43V18h8V17.43z"/></svg>'  # noqa



"""Class implementation for the SVG text's leading mix-in.
"""

from typing import Dict
from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class SvgTextLeadingMixIn(
    VariableNameMixIn,
    RevertMixIn,
):
    _leading: float = 1.5

    @property
    @add_debug_info_setting(module_name=__name__)
    def leading(self) -> Number:
        """
        Get a current leading setting.

        Returns
        -------
        leading : Number
            A current leading setting.
        """
        from apysc._expression import expression_data_util
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self,
            value_identifier="leading",
        )
        leading: Number = Number(self._leading, variable_name_suffix=suffix)
        expression: str = (
            f'{leading.variable_name} = {self.variable_name}.font("leading");'
        )
        expression_data_util.append_js_expression(expression=expression)
        return leading

    @leading.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def leading(self, value: Number) -> None:
        """
        Set a leading setting.

        Parameters
        ----------
        value : Number
            A leading setting.
        """
        from apysc._expression import expression_data_util

        self._leading = value._value
        expression: str = (
            f'{self.variable_name}.font("leading", {value.variable_name});'
        )
        expression_data_util.append_js_expression(expression=expression)

    _leading_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_leading_snapshots",
            value=self._leading,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._leading = self._get_snapshot_val_if_exists(
            current_value=self._leading,
            snapshot_dict=self._leading_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is for the translation mapping data of the
following document:

Document file: timer_complete.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class timer_complete interface": "# Timer クラスの timer_complete インターフェイス",
    ##################################################
    "This page explains the `Timer` class `timer_complete` method interface.": "このページでは`Timer`クラスの`timer_complete`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `timer_complete` method interface binds a new handler that a timer calls when it is complete. For instance, if the `repeat_count` argument is 100, it calls this handler when a timer count reaches 100 times.": "`timer_complete`メソッドのインターフェイスはタイマーが終了（完了）した際に呼ばれるハンドラを設定します。例えば`repeat_count`の引数に100を設定した場合そのハンドラはタイマーのカウントが100回に到達したタイミングで呼ばれます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `timer_complete` method has the same interface as the other event binding interface (arguments of the `handler` callable and `options` dictionary).": "`timer_complete`メソッドは他のイベント設定のインターフェイスと同様に関数やメソッドのハンドラとしての`handler`の引数とハンドラに渡すオプションのパラメーターとしての`options`引数の辞書を受け付けます。",  # noqa
    ##################################################
    "The following example starts the first timer (rotating the left-side rectangle) when you click the rectangle. If that one completes, then the second timer starts:": "以下のコード例では四角をクリックした際に1つ目の左側の四角に対する回転設定用のタイマーを開始しています。1つ目のタイマーが終了したタイミングで2つ目のタイマーを開始しています:",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectsOptions(TypedDict):\n    rectangle_1: ap.Rectangle\n    rectangle_2: ap.Rectangle\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: _RectsOptions) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click(on_click)\n    rectangle_1: ap.Rectangle = options["rectangle_1"]\n    rectangle_2: ap.Rectangle = options["rectangle_2"]\n    options_: _RectOptions = {"rectangle": rectangle_1}\n    timer_1: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_\n    )\n    options_ = {"rectangle": rectangle_2}\n    timer_1.timer_complete(handler=on_timer_1_complete, options=options_)\n    timer_1.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_1_complete(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the first time calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_2: ap.Rectangle = options["rectangle"]\n    options_: _RectOptions = {"rectangle": rectangle_2}\n    timer_2: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_\n    )\n    timer_2.start()\n\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions: _RectsOptions = {"rectangle_1": rectangle_1, "rectangle_2": rectangle_2}\nsprite.click(handler=on_click, options=options)\n\nap.save_overall_html(dest_dir_path="timer_complete_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectsOptions(TypedDict):\n    rectangle_1: ap.Rectangle\n    rectangle_2: ap.Rectangle\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: _RectsOptions) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click(on_click)\n    rectangle_1: ap.Rectangle = options["rectangle_1"]\n    rectangle_2: ap.Rectangle = options["rectangle_2"]\n    options_: _RectOptions = {"rectangle": rectangle_1}\n    timer_1: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_\n    )\n    options_ = {"rectangle": rectangle_2}\n    timer_1.timer_complete(handler=on_timer_1_complete, options=options_)\n    timer_1.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_timer_1_complete(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the first time calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_2: ap.Rectangle = options["rectangle"]\n    options_: _RectOptions = {"rectangle": rectangle_2}\n    timer_2: ap.Timer = ap.Timer(\n        handler=on_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_\n    )\n    timer_2.start()\n\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions: _RectsOptions = {"rectangle_1": rectangle_1, "rectangle_2": rectangle_2}\nsprite.click(handler=on_click, options=options)\n\nap.save_overall_html(dest_dir_path="timer_complete_basic_usage/")\n```',  # noqa
    ##################################################
    "## timer_complete API": "## timer_complete API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add a timer complete event listener setting.<hr>": "タイマー終了時のイベントハンドラの設定を追加します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A callable that a timer calls when complete.": "  - タイマー終了時に呼ばれる関数もしくはメソッド。",
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> def on_timer_complete(e: ap.TimerEvent, options: RectOptions) -> None:\n...     ap.trace("Timer completed!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, options=options)\n>>> _ = timer.timer_complete(on_timer_complete)\n>>> timer.start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> def on_timer_complete(e: ap.TimerEvent, options: RectOptions) -> None:\n...     ap.trace("Timer completed!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, options=options)\n>>> _ = timer.timer_complete(on_timer_complete)\n>>> timer.start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
}



"""`trace` (console.log expression) interface implementations
"""

import inspect
from inspect import FrameInfo
from types import FrameType
from typing import Any
from typing import List
from typing import Optional

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting


@add_debug_info_setting(module_name=__name__)
def trace(*args: Any) -> None:
    """
    Display arguments information to console.
    This function saves a JavaScript `console.log` expression.

    Notes
    -----
    The `ap.print` function is the alias of the `trace` function.

    Parameters
    ----------
    *args : list
        Any arguments to display to console.

    References
    ----------
    - Trace interface
        - https://simon-ritchie.github.io/apysc/en/trace.html

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.trace("Int value is:", int_val)
    """
    from apysc._expression import expression_data_util
    from apysc._string import string_util
    from apysc._type.variable_name_mixin import VariableNameMixIn

    expression: str = "console.log("
    arg_strs: List[str] = []
    for arg in args:
        if isinstance(arg, VariableNameMixIn):
            arg_strs.append(f"{arg.variable_name}")
            continue
        arg = string_util.escape_str(string=str(arg))
        arg = string_util.escape_double_quotation(string=arg)
        arg_strs.append(f'"{arg}"')
    expression += ", ".join(arg_strs)

    func_caller_info: str = _get_func_callers_info()
    expression += f', "{func_caller_info}"'

    expression += ");"
    expression_data_util.append_js_expression(expression=expression)


_temporary_outer_frames_index_adjustments: Optional[int] = None


class TemporaryOuterFramesIndexAdjustment:
    _temporary_outer_frames_index_adjustments: int

    @final
    def __init__(self, *, temporary_outer_frames_index_adjustments: int) -> None:
        """
        The class for the trace's temporary outer frames index setting.

        Parameters
        ----------
        temporary_outer_frames_index_adjustments : int
            A temporary outer frames index setting to set.
        """
        self._temporary_outer_frames_index_adjustments = (
            temporary_outer_frames_index_adjustments
        )

    @final
    def __enter__(self) -> None:
        """
        Enter and set the temporary outer frames index setting.
        """
        global _temporary_outer_frames_index_adjustments
        _temporary_outer_frames_index_adjustments = (
            self._temporary_outer_frames_index_adjustments
        )

    @final
    def __exit__(self, *args: Any) -> None:
        """
        Exit and revert the temporary outer frames index setting.
        """
        global _temporary_outer_frames_index_adjustments
        _temporary_outer_frames_index_adjustments = None


# Index count: _get_func_callers_info's function + trace + decorator + caller = 3
DEFAULT_OUTER_FRAMES_INDEX: int = 3


def _get_outer_frames_index() -> int:
    """
    Get the trace's outer frames' index setting.

    Returns
    -------
    outer_frames_index : int
        The trace's outer frames' index setting.
    """
    global _temporary_outer_frames_index_adjustments
    if _temporary_outer_frames_index_adjustments is None:
        return DEFAULT_OUTER_FRAMES_INDEX
    return _temporary_outer_frames_index_adjustments


def _get_func_callers_info() -> str:
    """
    Get a function caller's information.

    Returns
    -------
    func_caller_info : str
        A function caller's information, such as the caller's name,
        module name, and line number.
    """
    outer_frames_index: int = _get_outer_frames_index()
    current_frame: Optional[FrameType] = inspect.currentframe()
    outer_frames: List[FrameInfo] = inspect.getouterframes(frame=current_frame)
    file_name: str = outer_frames[outer_frames_index].filename
    file_name = file_name.rsplit("/", maxsplit=1)[-1]
    lineno: int = outer_frames[outer_frames_index].lineno
    function: str = outer_frames[outer_frames_index].function
    if function != "<module>":
        func_caller_info: str = (
            f"\\nCalled from: {function}, file name: {file_name}, line number: {lineno}"
        )
    else:
        func_caller_info = f"\\nCalled from: {file_name}, line number: {lineno}"
    return func_caller_info



"""The class implementation for the outlined `admin_panel_settings` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialAdminPanelSettingsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `admin_panel_settings` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><circle cx="17" cy="15.5" fill-rule="evenodd" r="1.12"/><path d="M17,17.5c-0.73,0-2.19,0.36-2.24,1.08c0.5,0.71,1.32,1.17,2.24,1.17 s1.74-0.46,2.24-1.17C19.19,17.86,17.73,17.5,17,17.5z" fill-rule="evenodd"/><path d="M18,11.09V6.27L10.5,3L3,6.27v4.91c0,4.54,3.2,8.79,7.5,9.82 c0.55-0.13,1.08-0.32,1.6-0.55C13.18,21.99,14.97,23,17,23c3.31,0,6-2.69,6-6C23,14.03,20.84,11.57,18,11.09z M11,17 c0,0.56,0.08,1.11,0.23,1.62c-0.24,0.11-0.48,0.22-0.73,0.3c-3.17-1-5.5-4.24-5.5-7.74v-3.6l5.5-2.4l5.5,2.4v3.51 C13.16,11.57,11,14.03,11,17z M17,21c-2.21,0-4-1.79-4-4c0-2.21,1.79-4,4-4s4,1.79,4,4C21,19.21,19.21,21,17,21z" fill-rule="evenodd"/></g></g></svg>'  # noqa



"""The class implementation for the outlined `toll` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTollOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `toll` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15 4c-4.42 0-8 3.58-8 8s3.58 8 8 8 8-3.58 8-8-3.58-8-8-8zm0 14c-3.31 0-6-2.69-6-6s2.69-6 6-6 6 2.69 6 6-2.69 6-6 6zM3 12c0-2.61 1.67-4.83 4-5.65V4.26C3.55 5.15 1 8.27 1 12s2.55 6.85 6 7.74v-2.09c-2.33-.82-4-3.04-4-5.65z"/></svg>'  # noqa



"""The class implementation for the outlined `verified_user` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVerifiedUserOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `verified_user` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 1L3 5v6c0 5.55 3.84 10.74 9 12 5.16-1.26 9-6.45 9-12V5l-9-4zm7 10c0 4.52-2.98 8.69-7 9.93-4.02-1.24-7-5.41-7-9.93V6.3l7-3.11 7 3.11V11zm-11.59.59L6 13l4 4 8-8-1.41-1.42L10 14.17z"/></svg>'  # noqa



"""The class implementation for the `lightbulb` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLightbulbIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `lightbulb` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M9 21c0 .5.4 1 1 1h4c.6 0 1-.5 1-1v-1H9v1zm3-19C8.1 2 5 5.1 5 9c0 2.4 1.2 4.5 3 5.7V17c0 .5.4 1 1 1h6c.6 0 1-.5 1-1v-2.3c1.8-1.3 3-3.4 3-5.7 0-3.9-3.1-7-7-7z"/></svg>'  # noqa



"""The static class implementation for the material design settings.
"""

from typing import List
from typing import Optional
from typing import Union

from apysc._converter.list_of_strs_to_array import list_of_strs_to_array_of_string
from apysc._material_design.color.material_brightness import MaterialBrightness
from apysc._material_design.color.material_color_scheme import MaterialColorScheme
from apysc._type.array import Array
from apysc._type.boolean import Boolean
from apysc._type.string import String


class MaterialSettings:

    @classmethod
    def _reset_settings(cls) -> None:
        """
        Reset all settings of this class.
        """
        cls._delete_current_brightness_string_attr()
        cls._light_color_scheme = None
        cls._dark_color_scheme = None
        if hasattr(cls, "_light_color_scheme_str"):
            del cls._light_color_scheme_str
        if hasattr(cls, "_dark_color_scheme_str"):
            del cls._dark_color_scheme_str
        if hasattr(cls, "_color_scheme_setting_is_enabled"):
            del cls._color_scheme_setting_is_enabled
        cls._font_family = None

    _light_color_scheme: Optional[MaterialColorScheme] = None

    @classmethod
    def _initialize_attrs(cls) -> None:
        """
        Initialize each attribute if it has not been initialized yet.
        """
        cls._initialize_current_brightness_string_if_not_initialized()
        if not hasattr(cls, "_light_color_scheme_str"):
            cls._light_color_scheme_str = String(
                value=MaterialBrightness.LIGHT.value,
                variable_name_suffix="light_color_scheme_str",
            )
        if not hasattr(cls, "_dark_color_scheme_str"):
            cls._dark_color_scheme_str = String(
                value=MaterialBrightness.DARK.value,
                variable_name_suffix="dark_color_scheme_str",
            )
        cls._initialize_color_scheme_setting_is_enabled_if_not_initialized()

    @classmethod
    def get_light_color_scheme(cls) -> Optional[MaterialColorScheme]:
        """
        Get a light color scheme setting.

        Returns
        -------
        color_scheme : MaterialColorScheme or None
            A light color scheme setting. If the color scheme is not set,
            this property becomes None.
        """
        cls._initialize_attrs()
        return cls._light_color_scheme

    @classmethod
    def set_light_color_scheme(cls, *, color_scheme: MaterialColorScheme) -> None:
        """
        Set a color scheme setting.

        Parameters
        ----------
        color_scheme : MaterialColorScheme
            Color scheme to set.

        Raises
        ------
        ValueError
            If you call this class method multiple times.
        """
        cls._initialize_attrs()
        cls._enable_color_scheme_setting()
        if cls._light_color_scheme is not None:
            raise ValueError(
                "The `set_light_color_scheme` class method can only be called once."
            )
        cls._light_color_scheme = color_scheme

    _dark_color_scheme: Optional[MaterialColorScheme] = None

    @classmethod
    def get_dark_color_scheme(cls) -> Optional[MaterialColorScheme]:
        """
        Get a dark color scheme setting.

        Returns
        -------
        color_scheme : MaterialColorScheme or None
            A dark color scheme setting. If the color scheme is not set,
            this property becomes None.
        """
        cls._initialize_attrs()
        return cls._dark_color_scheme

    @classmethod
    def set_dark_color_scheme(cls, *, color_scheme: MaterialColorScheme) -> None:
        """
        Set a dark color scheme setting.

        Parameters
        ----------
        color_scheme : MaterialColorScheme
            Color scheme to set.

        Raises
        ------
        ValueError
            If you call this class method multiple times.
        """
        cls._initialize_attrs()
        cls._enable_color_scheme_setting()
        if cls._dark_color_scheme is not None:
            raise ValueError(
                "The `set_dark_color_scheme` class method can only be called once."
            )
        cls._dark_color_scheme = color_scheme

    _current_brightness_string: String

    @classmethod
    def _initialize_current_brightness_string_if_not_initialized(cls) -> None:
        """
        Initialize the `_current_brightness_string` attribute string
        if it has not been initialized yet.
        """
        if hasattr(cls, "_current_brightness_string"):
            return
        cls._current_brightness_string = String(
            value=MaterialBrightness.LIGHT.value,
            variable_name_suffix="current_brightness_string",
        )

    @classmethod
    def _delete_current_brightness_string_attr(cls) -> None:
        """
        Delete the `_current_brightness_string` attribute.
        """
        if hasattr(cls, "_current_brightness_string"):
            del cls._current_brightness_string

    @classmethod
    def switch_to_light_color_scheme(cls) -> None:
        """
        Switch the current color scheme to the light color scheme.
        """
        cls._initialize_attrs()
        cls._current_brightness_string.value = MaterialBrightness.LIGHT.value

    @classmethod
    def switch_to_dark_color_scheme(cls) -> None:
        """
        Switch the current color scheme to the dark color scheme.
        """
        cls._initialize_attrs()
        cls._current_brightness_string.value = MaterialBrightness.DARK.value

    _light_color_scheme_str: String

    @classmethod
    def current_color_scheme_is_light_color_scheme(cls) -> Boolean:
        """
        Get whether the current color scheme is the light color scheme or not.

        Returns
        -------
        result : Boolean
            If the current color scheme is the light color scheme, then
            this method returns True.
        """
        cls._initialize_attrs()
        return cls._current_brightness_string == cls._light_color_scheme_str

    _dark_color_scheme_str: String

    @classmethod
    def current_color_scheme_is_dark_color_scheme(cls) -> Boolean:
        """
        Get a boolean whether the current color scheme
        is the dark color scheme or not.

        Returns
        -------
        result : Boolean
            If the current color scheme is the dark color scheme, then
            this method returns True.
        """
        cls._initialize_attrs()
        return cls._current_brightness_string == cls._dark_color_scheme_str

    _color_scheme_setting_is_enabled: Boolean

    @classmethod
    def _initialize_color_scheme_setting_is_enabled_if_not_initialized(cls) -> None:
        """
        Initialize the `_color_scheme_setting_is_enabled` attribute if it has
        not been initialized yet.
        """
        if hasattr(cls, "_color_scheme_setting_is_enabled"):
            return
        cls._color_scheme_setting_is_enabled = Boolean(
            False, variable_name_suffix="color_scheme_setting_is_enabled"
        )

    @classmethod
    def _enable_color_scheme_setting(cls) -> None:
        """
        Enable the color scheme setting.
        """
        cls._initialize_attrs()
        if cls._color_scheme_setting_is_enabled._value:
            return
        cls._color_scheme_setting_is_enabled.value = True
        cls._color_scheme_setting_is_enabled.disable_value_modification()

    @classmethod
    def color_scheme_setting_is_enabled(cls) -> Boolean:
        """
        Get a boolean whether the color scheme setting is enabled or not.

        Returns
        -------
        result : Boolean
            If the color scheme setting is enabled, then this method
            returns True.
        """
        cls._initialize_attrs()
        return cls._color_scheme_setting_is_enabled

    _font_family: Optional[Array[String]] = None

    @classmethod
    def set_font_family(cls, *, font_family: Union[Array[String], List[str]]) -> None:
        """
        Set a font-family setting.

        Parameters
        ----------
        font_family : Union[Array[String], List[str]]
            A font-family setting.
        """
        cls._initialize_attrs()
        font_family_: Optional[Array[String]] = list_of_strs_to_array_of_string(
            optional_list_or_arr=font_family
        )
        cls._font_family = font_family_

    @classmethod
    def get_font_family(cls) -> Optional[Array[String]]:
        """
        Get a font-family setting.

        Returns
        -------
        font_family : Array[String] or None
            A font-family setting.
        """
        cls._initialize_attrs()
        if cls._font_family is None:
            return None
        return cls._font_family.copy()



"""The mix-in class implementation for the `SvgText`'s `_set_text_value` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String


class SvgTextSetTextValueMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_text_value(self, *, text: Union[str, String]) -> None:
        """
        Set a text value.

        Parameters
        ----------
        text : Union[str, String]
            A target text.
        """
        from apysc._display.svg_text_text_mixin import SvgTextTextMixIn
        from apysc._string import string_util
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if not isinstance(self, SvgTextTextMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextTextMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if isinstance(text, str):
            text = string_util.escape_str(string=text)
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self,
                value_identifier="text",
            )
            text_: String = String(
                text,
                variable_name_suffix=suffix,
            )
        else:
            text_ = text
        self.text = text_



"""The class implementation for the outlined `invert_colors` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialInvertColorsOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `invert_colors` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M24 0H0v24h24V0z" fill="none"/><path d="M17.66 7.93L12 2.27 6.34 7.93c-3.12 3.12-3.12 8.19 0 11.31C7.9 20.8 9.95 21.58 12 21.58s4.1-.78 5.66-2.34c3.12-3.12 3.12-8.19 0-11.31zM12 19.59c-1.6 0-3.11-.62-4.24-1.76C6.62 16.69 6 15.19 6 13.59s.62-3.11 1.76-4.24L12 5.1v14.49z"/></svg>'  # noqa



"""The class implementation for the `cell_wifi` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCellWifiIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `cell_wifi` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M18,9.98L6,22h12h4V5.97L18,9.98z M20,20h-2v-7.22l2-2V20z M5.22,7.22L3.93,5.93c3.9-3.91,10.24-3.91,14.15,0l-1.29,1.29 C13.6,4.03,8.41,4.03,5.22,7.22z M12.93,11.07L11,13l-1.93-1.93C10.14,10.01,11.86,10.01,12.93,11.07z M14.22,9.79 c-1.78-1.77-4.66-1.77-6.43,0L6.5,8.5c2.48-2.48,6.52-2.48,9,0L14.22,9.79z"/></svg>'  # noqa



"""Class implementation for the string class.
"""

from typing import Any
from typing import Dict
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._type.copy_mixin import CopyMixIn
from apysc._type.hashable_interface import HashableInterface
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string_apply_max_num_of_decimal_places_mixin import (
    StringApplyMaxNumOfDecimalPlacesMixIn,
)
from apysc._type.string_length_mixin import StringLengthMixIn
from apysc._type.string_lower_mixin import StringLowerMixIn
from apysc._type.string_lstrip_mixin import StringLStripMixIn
from apysc._type.string_rstrip_mixin import StringRStripMixIn
from apysc._type.string_slice_mixin import StringSliceMixIn
from apysc._type.string_split_mixin import StringSplitMixIn
from apysc._type.string_strip_mixin import StringStripMixIn
from apysc._type.string_upper_mixin import StringUpperMixIn
from apysc._type.string_zfill_mixin import StringZfillMixIn
from apysc._type.to_number_mixin import ToNumberMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class String(
    StringSplitMixIn,
    StringLStripMixIn,
    StringStripMixIn,
    StringRStripMixIn,
    CopyMixIn["String"],
    RevertMixIn,
    CustomEventMixIn,
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
    InitializeWithBaseValueInterface,
    HashableInterface,
    ToNumberMixIn,
    StringLengthMixIn,
    StringApplyMaxNumOfDecimalPlacesMixIn,
    StringZfillMixIn,
    StringLowerMixIn,
    StringUpperMixIn,
    StringSliceMixIn,
):
    """
    String class for apysc library.

    Notes
    -----
    The `Str` class is the alias of `String`.

    References
    ----------
    - String
        - https://simon-ritchie.github.io/apysc/en/string.html
    - String class comparison operations
        - https://simon-ritchie.github.io/apysc/en/string_comparison_operations.html  # noqa
    - String class addition and multiplication operations
        - https://simon-ritchie.github.io/apysc/en/string_addition_and_multiplication.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> string: ap.String = ap.String("Hello")
    >>> string
    String("Hello")

    >>> string += " World!"
    >>> string
    String("Hello World!")

    >>> string.value = "World!"
    >>> string
    String("World!")

    >>> string.value = "Hello!"
    >>> string *= 3
    >>> string
    String("Hello!Hello!Hello!")
    """

    _initial_value: Union[str, "String"]
    _value: str

    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[str, "String"],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        String class for apysc library.

        Parameters
        ----------
        value : String or str
            Initial string value.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        Notes
        -----
        The `Str` class is the alias of `String`.

        References
        ----------
        - String
            - https://simon-ritchie.github.io/apysc/en/string.html
        - String class comparison operations
            - https://simon-ritchie.github.io/apysc/en/string_comparison_operations.html  # noqa
        - String class addition and multiplication operations
            - https://simon-ritchie.github.io/apysc/en/string_addition_and_multiplication.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> string: ap.String = ap.String("Hello")
        >>> string
        String("Hello")

        >>> string += " World!"
        >>> string
        String("Hello World!")
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            value = _escape_str_value(value=value)
            self._variable_name_suffix = variable_name_suffix
            TYPE_NAME: str = var_names.STRING
            self._initial_value = value
            self._type_name = TYPE_NAME
            self._value = self._get_str_value(value=value)
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        from apysc._expression import expression_data_util

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        expression: str = f"{self.variable_name} = "
        if isinstance(self._initial_value, String):
            expression += f"{self._initial_value.variable_name};"
        else:
            expression += f'"{self._value}";'
        return expression

    @final
    def _get_str_value(self, *, value: Union[str, "String"]) -> str:
        """
        Get a (Python's) str value from a specified value.

        Parameters
        ----------
        value : String or str
            Target string value.

        Returns
        -------
        value : str
            Python's built-in str value.
        """
        if isinstance(value, String):
            return value._value
        return str(value)

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Union[str, "String"]:
        """
        Get a current string value.

        Returns
        -------
        value : str
            Current string value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> string: ap.String = ap.String("Hello")
        >>> string.value = "World!"
        >>> string.value
        'World!'
        """
        return self._value

    @value.setter
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def value(self, value: Union[str, "String"]) -> None:
        """
        Set string value.

        Parameters
        ----------
        value : String or str
            Any string value to set.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa
        """
        self._value = self._get_str_value(value=value)
        self._append_value_setter_expression(value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_setter_expression(self, *, value: Union[str, "String"]) -> None:
        """
        Append value's setter expression.

        Parameters
        ----------
        value : String or str
            Any string value to set.
        """
        from apysc._expression import expression_data_util

        expression: str = f"{self.variable_name} = "
        if isinstance(value, String):
            expression += f"{value.variable_name};"
        else:
            expression += f'"{value}";'
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __add__(self, other: Union[str, "String"]) -> "String":
        """
        Method for addition (string concatenation).

        Parameters
        ----------
        other : String or str
            The other string value to concatenate.

        Returns
        -------
        result : String
            Concatenated result string.
        """
        if isinstance(other, String):
            value: str = self._value + other._value
        else:
            value = self._value + other  # type: ignore
        result: String = self.copy()
        result._value = value
        self._append_addition_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_addition_expression(
        self, *, result: VariableNameMixIn, other: Union[str, "String"]
    ) -> None:
        """
        Append addition (string concatenation) expression.

        Parameters
        ----------
        result : String
            Addition result value.
        other : String or str
            The other string value to concatenate.
        """
        from apysc._expression import expression_data_util
        from apysc._type.value_util import get_value_str_for_expression

        right_value: str = get_value_str_for_expression(value=other)
        expression: str = (
            f"var {result.variable_name} = " f"{self.variable_name} + {right_value};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __mul__(self, other: Union[int, Any]) -> "String":
        """
        Method for multiplication (string repetition).

        Parameters
        ----------
        other : Int or int
            String repetition number.

        Returns
        -------
        result : String
            Repeated result string.
        """
        from apysc._type.int import Int

        if isinstance(other, Int):
            value: int = other.value  # type: ignore
        else:
            value = other
        result: String = self.copy()
        result._value = result._value * value
        self._append_multiplication_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_multiplication_expression(
        self, *, result: VariableNameMixIn, other: Union[int, Any]
    ) -> None:
        """
        Append multiplication (string repetition) expression.

        Parameters
        ----------
        result : String
            Multiplication result value.
        other : Int or int
            String repetition number.
        """
        from apysc._expression import expression_data_util
        from apysc._type.int import Int

        expression: str = f'var {result.variable_name} = "";'
        expression += "\nfor (var i = 0; i < "
        if isinstance(other, Int):
            expression += f"{other.variable_name}"
        else:
            expression += f"{other}"
        expression += "; i++) {"
        expression += f"\n  {result.variable_name} += {self.variable_name};"
        expression += "\n}"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __iadd__(self, other: Union[str, "String"]) -> Any:
        """
        Method for incremental addition (string concatenation).

        Parameters
        ----------
        other : String or str
            The other string value to concatenate.

        Returns
        -------
        result : String
            Concatenated result string.
        """
        from apysc._expression import expression_variables_util

        result: String = self + other
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result
        )
        result.variable_name = self.variable_name
        return result

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __imul__(self, other: Union[int, Any]) -> Any:
        """
        Method for incremental multiplication (string repetition).

        Parameters
        ----------
        other : Int or int
            String repetition number.

        Returns
        -------
        result : String
            Repetition result string.
        """
        from apysc._expression import expression_variables_util

        result: String = self * other
        expression_variables_util.append_substitution_expression(
            left_value=self, right_value=result
        )
        result.variable_name = self.variable_name
        return result

    def __str__(self) -> str:
        """
        Method for str conversion.

        Returns
        -------
        result : str
            Python builtins str value.
        """
        if not hasattr(self, "_value"):
            return ""
        return self._value

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Method for equal comparison.

        Parameters
        ----------
        other : *
            Any value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. If the equal value of a String
            or str is specified, this interface returns True.
        """
        from apysc._type.boolean import Boolean

        if isinstance(other, str):
            result: Boolean = Boolean(
                self._value == other, variable_name_suffix=self._variable_name_suffix
            )
        elif isinstance(other, String):
            result = Boolean(
                self._value == other._value,
                variable_name_suffix=self._variable_name_suffix,
            )
        else:
            result = Boolean(False)
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_eq_expression(result=result, other=other)
        return result

    @final
    def _convert_other_val_to_string(self, *, other: Any) -> Any:
        """
        Convert a comparison other value to a String if it is
        a string value.

        Parameters
        ----------
        other : *
            Other comparison value.

        Returns
        -------
        converted_val : *
            Converted value. If the other value is a string,
            this interface converts it to a String value.
            This interface returns the other type value
            directly (not to be converted).
        """
        if isinstance(other, str):
            return String(other, variable_name_suffix=self._variable_name_suffix)
        return other

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_eq_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __eq__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} === {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Method for not equal comparison.

        Parameters
        ----------
        other : *
            Any value to compare.

        Returns
        -------
        result : Boolean
            Comparison result. If a specified value is not
            the equal value of a String or str, this interface
            returns True.
        """
        from apysc._type.boolean import Boolean

        if isinstance(other, str):
            result: Boolean = Boolean(
                self._value != other, variable_name_suffix=self._variable_name_suffix
            )
        elif isinstance(other, String):
            result = Boolean(
                self._value != other._value,
                variable_name_suffix=self._variable_name_suffix,
            )
        else:
            result = Boolean(True, variable_name_suffix=self._variable_name_suffix)
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_ne_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_ne_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __ne__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} !== {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __lt__(self, other: Union[str, Any]) -> Any:
        """
        Method for less than comparison.

        Parameters
        ----------
        other : String or str
            String value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        from apysc._type.boolean import Boolean

        value: str = self._get_str_value(value=other)
        result: Boolean = Boolean(
            self._value < value, variable_name_suffix=self._variable_name_suffix
        )
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_lt_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_lt_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __lt__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} < {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __le__(self, other: Union[str, Any]) -> Any:
        """
        Method for less than or equal comparison.

        Parameters
        ----------
        other : String or str
            String value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        from apysc._type.boolean import Boolean

        value: str = self._get_str_value(value=other)
        result: Boolean = Boolean(
            self._value <= value, variable_name_suffix=self._variable_name_suffix
        )
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_le_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_le_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __le__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} <= {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __gt__(self, other: Union[str, Any]) -> Any:
        """
        Method for greater than comparison.

        Parameters
        ----------
        other : String or str
            String value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        from apysc._type.boolean import Boolean

        value: str = self._get_str_value(value=other)
        result: Boolean = Boolean(
            self._value > value, variable_name_suffix=self._variable_name_suffix
        )
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_gt_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_gt_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __gt__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} > {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __ge__(self, other: Union[str, Any]) -> Any:
        """
        Method for greater than or equal comparison.

        Parameters
        ----------
        other : String or str
            String value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        from apysc._type.boolean import Boolean

        value: str = self._get_str_value(value=other)
        result: Boolean = Boolean(
            self._value >= value, variable_name_suffix=self._variable_name_suffix
        )
        other = self._convert_other_val_to_string(other=other)
        if isinstance(other, VariableNameMixIn):
            self._append_ge_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_ge_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __ge__ method expression.

        Parameters
        ----------
        result : Boolean
            Result boolean value.
        other : VariableNameMixIn
            Other value to compare.
        """
        from apysc._expression import expression_data_util

        expression: str = (
            f"{result.variable_name} = "
            f"{self.variable_name} >= {other.variable_name};"
        )
        expression_data_util.append_js_expression(expression=expression)

    def __int__(self) -> int:
        """
        Method for integer conversion.

        Returns
        -------
        result : int
            Converted integer value.
        """
        result: int = int(self._value)
        return result

    def __float__(self) -> float:
        """
        Method for float conversion.

        Returns
        -------
        result : float
            Converted float value.
        """
        result: float = float(self._value)
        return result

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = 'String("")'
        else:
            repr_str = f'String("{self._value}")'
        return repr_str

    _value_snapshots: Optional[Dict[str, str]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_value_snapshots", value=self._value, snapshot_name=snapshot_name
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._value = self._get_snapshot_val_if_exists(
            current_value=self._value,
            snapshot_dict=self._value_snapshots,
            snapshot_name=snapshot_name,
        )

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "String":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        str_value : String
            An initialized string value.
        """
        return String("")

    @final
    def __hash__(self) -> int:
        """
        Get a hashed value.

        Returns
        -------
        hashed_value : int
            A hashed value.
        """
        return hash(self._value)


def _escape_str_value(*, value: Union[str, "String"]) -> Union[str, "String"]:
    """
    Escape a specified string value.

    Notes
    -----
    This function only applies to escape only when a specified value type
    is `str`.

    Parameters
    ----------
    value : Union[str, "String"]
        A specified string value.

    Returns
    -------
    value : Unjion[str, "String"]
        An escaped string value.
    """
    from apysc._string import string_util

    if isinstance(value, str):
        value = string_util.escape_str(string=value)
        return value

    return value



"""The class implementation for the `unpublished` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialUnpublishedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `unpublished` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M21.19,21.19L2.81,2.81L1.39,4.22l2.27,2.27C2.61,8.07,2,9.96,2,12c0,5.52,4.48,10,10,10c2.04,0,3.93-0.61,5.51-1.66 l2.27,2.27L21.19,21.19z M10.59,16.6l-4.24-4.24l1.41-1.41l2.83,2.83l0.18-0.18l1.41,1.41L10.59,16.6z M13.59,10.76l-7.1-7.1 C8.07,2.61,9.96,2,12,2c5.52,0,10,4.48,10,10c0,2.04-0.61,3.93-1.66,5.51l-5.34-5.34l2.65-2.65l-1.41-1.41L13.59,10.76z"/></svg>'  # noqa



"""Class implementation for the center y-coordinate mix-in.
"""

from typing import Dict
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._animation.animation_cy_mixin import AnimationCyMixIn
from apysc._display.y_interface import YInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class CyMixIn(
    YInterface,
    VariableNameSuffixAttrOrVarMixIn,
    AnimationCyMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):
    @final
    def _initialize_y_if_not_initialized(self) -> None:
        """
        Initialize _y attribute if this instance does not
        initialize it yet.
        """
        if hasattr(self, "_y"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="y")
        self._y = Number(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_y_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_y_attr_linking_setting(self) -> None:
        """
        Append y attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(new_attr=self._y, attr_name="y")
        self._append_attr_to_linking_stack(attr=self._y, attr_name="y")

    @property
    @add_debug_info_setting(module_name=__name__)
    def y(self) -> Number:
        """
        Get a center y-coordinate.

        Returns
        -------
        y : Number
            Center y-coordinate.

        References
        ----------
        - Display object x and y interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_x_and_y.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"), alpha=0.5)
        >>> circle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=50)
        >>> circle.y = ap.Number(120)
        >>> circle.y
        Number(120.0)
        """
        from apysc._type import value_util

        self._initialize_y_if_not_initialized()
        y: Number = value_util.get_copy(value=self._y)
        return y

    @y.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def y(self, value: Number) -> None:
        """
        Update a center y-coordinate.

        Parameters
        ----------
        value : Number
            Center y-coordinate value.

        References
        ----------
        - Display object x and y interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_x_and_y.html  # noqa
        """
        self._y = value
        self._y._append_incremental_calc_substitution_expression()
        self._append_y_update_expression()

        self._append_y_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_y_update_expression(self) -> None:
        """
        Append y position updating expression.
        """
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        self._initialize_y_if_not_initialized()
        value_str: str = value_util.get_value_str_for_expression(value=self._y)
        expression: str = f"{self.variable_name}.cy({value_str});"
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _update_y_and_skip_appending_exp(self, *, y: Union[float, Number]) -> None:
        """
        Update y-coordinate and skip appending an expression.

        Parameters
        ----------
        y : float or Number
            Y-coordinate value.
        """
        if isinstance(y, Number):
            y_: Number = y
        else:
            suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="y")
            y_ = Number(y, variable_name_suffix=suffix)
        self._y = y_

    _y_snapshots: Optional[Dict[str, float]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_y_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_y_snapshots",
            value=float(self._y._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._y._value = self._get_snapshot_val_if_exists(
            current_value=self._y._value,
            snapshot_dict=self._y_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is for the translation mapping data of the
following document:

Document file: delete.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# delete interface": "# delete インターフェイス",
    ##################################################
    "This page explains the `delete` function interface.": "このページでは`delete`関数のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `delete` function deletes any instance, and it becomes an `undefined` object.": "`delete`関数は任意のインスタンスの削除を行います。削除された対象は`undefined`のオブジェクトになります。",  # noqa
    ##################################################
    "If an instance is a `DisplayObject` instance, this interface removes an instance from the stage.": "もし対象のインスタンスが`DisplayObject`のインスタンスであれば、このインターフェイスはそのインスタンスをステージから取り除きます。",  # noqa
    ##################################################
    "For example, this interface removes a `Sprite` or `Rectangle` instance.": "例えば`Sprite`や`Rectangle`などのインスタンスであればこのインターフェイスはそれらのインスタンスを取り除きます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "You can specify any apysc instance to the `delete` argument.": "`delete`関数の引数へは任意のapyscのインスタンスを指定することができます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_val: ap.Int = ap.Int(10)\nap.delete(int_val)\n```": "```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_val: ap.Int = ap.Int(10)\nap.delete(int_val)\n```",  # noqa
    ##################################################
    "If a specified instance is a `DisplayObject` instance, this function removes it from a stage.": "もし指定されたインスタンスが`DisplayObject`のインスタンスであれば、`delete`関数はそのインスタンスをステージから取り除きます。",  # noqa
    ##################################################
    "Also, it becomes an undefined object.": "また、削除されたインスタンスはundefinedのオブジェクトになります。",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The click event handler that a sprite calls.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Sprite]\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    ap.delete(sprite)\n    ap.assert_undefined(sprite)\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.click(on_click)\n\nap.save_overall_html(dest_dir_path="delete_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The click event handler that a sprite calls.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Sprite]\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    ap.delete(sprite)\n    ap.assert_undefined(sprite)\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.click(on_click)\n\nap.save_overall_html(dest_dir_path="delete_basic_usage/")\n```',  # noqa
    ##################################################
    "If you click the following rectangle, the `delete` function removes it from the stage.": "もし以下の四角をクリックすると、`delete`関数は対象のインスタンスをステージから取り除きます。",  # noqa
}



"""The class implementation for the `call` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialCallIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `call` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20.01 15.38c-1.23 0-2.42-.2-3.53-.56-.35-.12-.74-.03-1.01.24l-1.57 1.97c-2.83-1.35-5.48-3.9-6.89-6.83l1.95-1.66c.27-.28.35-.67.24-1.02-.37-1.11-.56-2.3-.56-3.53 0-.54-.45-.99-.99-.99H4.19C3.65 3 3 3.24 3 3.99 3 13.28 10.73 21 20.01 21c.71 0 .99-.63.99-1.18v-3.45c0-.54-.45-.99-.99-.99z"/></svg>'  # noqa



"""The mix-in class implementation for the `SvgText`'s `_set_delta_y` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class SvgTextSetDeltaYMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_delta_y(self, *, delta_y: Union[float, Number]) -> None:
        """
        Set a coordinate delta-y value.

        Parameters
        ----------
        delta_y : Union[float, Number]
            A delta-y value.
        """
        from apysc._display.svg_text_delta_y_mixin import SvgTextDeltaYMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if not isinstance(self, SvgTextDeltaYMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextDeltaYMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if not isinstance(delta_y, Number):
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self, value_identifier="delta_y"
            )
            delta_y_: Number = Number(delta_y, variable_name_suffix=suffix)
        else:
            delta_y_ = delta_y
        self.delta_y = delta_y_



"""The class implementation for the outlined `report_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialReportOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `report_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M9.1 5h5.8L19 9.1v5.8l-.22.22 1.42 1.41.8-.8V8.27L15.73 3H8.27l-.8.8 1.41 1.42z"/><circle cx="12" cy="16" r="1"/><path d="M13 9.33V7h-2v.33zM2.41 1.58L1 2.99l3.64 3.64L3 8.27v7.46L8.27 21h7.46l1.64-1.64L21.01 23l1.41-1.41L2.41 1.58zM14.9 19H9.1L5 14.9V9.1l1.05-1.05 9.9 9.9L14.9 19z"/></svg>'  # noqa



"""The class implementation for the outlined `invert_colors_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialInvertColorsOffOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `invert_colors_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12 5.1v4.05l7.4 7.4c1.15-2.88.59-6.28-1.75-8.61L12 2.27 8.56 5.71l1.41 1.41L12 5.1zm-7.6-.73L2.99 5.78l2.78 2.78c-2.54 3.14-2.35 7.75.57 10.68C7.9 20.8 9.95 21.58 12 21.58c1.78 0 3.56-.59 5.02-1.77l2.7 2.7 1.41-1.41L4.4 4.37zM12 19.59c-1.6 0-3.11-.62-4.24-1.76C6.62 16.69 6 15.19 6 13.59c0-1.32.43-2.56 1.21-3.59L12 14.79v4.8z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: math_clamp.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Math clamp interface": "# Math クラスの clamp インターフェイス",
    ##################################################
    "This page explains the `Math` class's `clamp` class method interface.": "このページでは`Math`クラスの`clamp`メソッドについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `clamp` method sets the value within a specified minimum and maximum range.": "`clamp`メソッドは指定された最小値と最大値の範囲内で値を設定します。",  # noqa
    ##################################################
    "For example, if the value is `20` and the minimum is `25`, the result value becomes `25`.": "例えば、もし値が`20`で最小値が`25`の場合、結果の値は`25`になります。",  # noqa
    ##################################################
    "Similarly, if the value is `20` and the maximum is `15`, the result value becomes `15`.": "同様に、もし値が`20`で最大値が`15`であれば、結果の値は`15`になります。",  # noqa
    ##################################################
    "If the value is `20` and the minimum and maximum are `15` and `25`, this method returns the `20` value as is.": "もし値が`20`で最小値と最大値が`15`と`25`であれば、このメソッドはそのまま`20`の値を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `clamp` method requires the `value`, `min_`, and `max_` arguments.": "`clamp`メソッドは`value`、`min_`、`max_`の引数を必要とします。",  # noqa
    ##################################################
    "Each argument and return value becomes the `ap.Int` or `ap.Number` type.": "各引数と返却値は`ap.Int`もしくは`ap.Number`型となります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=1,\n    stage_height=1,\n    stage_elem_id="stage",\n)\nvalue: ap.Int = ap.Int(20)\nresult: ap.Int = ap.Math.clamp(value=value, min_=ap.Int(25), max_=ap.Int(50))\nap.assert_equal(result, ap.Int(25))\n\nresult = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(15))\nap.assert_equal(result, ap.Int(15))\n\nresult = ap.Math.clamp(value=value, min_=ap.Int(15), max_=ap.Int(25))\nap.assert_equal(result, ap.Int(20))\n\nap.save_overall_html(dest_dir_path="math_clamp_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=1,\n    stage_height=1,\n    stage_elem_id="stage",\n)\nvalue: ap.Int = ap.Int(20)\nresult: ap.Int = ap.Math.clamp(value=value, min_=ap.Int(25), max_=ap.Int(50))\nap.assert_equal(result, ap.Int(25))\n\nresult = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(15))\nap.assert_equal(result, ap.Int(15))\n\nresult = ap.Math.clamp(value=value, min_=ap.Int(15), max_=ap.Int(25))\nap.assert_equal(result, ap.Int(20))\n\nap.save_overall_html(dest_dir_path="math_clamp_basic_usage/")\n```',  # noqa
    ##################################################
    "## clamp method API": "## clamp メソッドのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Sets the value within a specified minimum and maximum range. If the value is less than the minimum, this method returns the minimum value. If the value is greater than the maximum, this method returns the maximum value.<hr>": "指定された最小値と最大値の範囲内で値を設定します。もし値が最小値未満であればこのメソッドは最小値を返却します。もし値が最大値よりも大きければこのメソッドは最大値を返却します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: _ValueType": "- `value`: _ValueType",
    ##################################################
    "  - Target `Int` or `Number` value.": "  - 対象の`Int`もしくは`Number`型の値。",
    ##################################################
    "- `min_`: _ValueType": "- `min_`: _ValueType",
    ##################################################
    "  - Minimum value.": "  - 最小値。",
    ##################################################
    "- `max_`: _ValueType": "- `max_`: _ValueType",
    ##################################################
    "  - Maximum value.": "  - 最大値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: _ValueType": "- `result`: _ValueType",
    ##################################################
    "  - Clamped value.": "  - 最小値と最大値範囲の反映後の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> value: ap.Int = ap.Int(5)\n>>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))\n>>> value\nInt(10)\n\n>>> value = ap.Int(25)\n>>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))\n>>> value\nInt(20)\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> value: ap.Int = ap.Int(5)\n>>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))\n>>> value\nInt(10)\n\n>>> value = ap.Int(25)\n>>> value = ap.Math.clamp(value=value, min_=ap.Int(10), max_=ap.Int(20))\n>>> value\nInt(20)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# AnimationEvent class": "# AnimationEvent クラス",
    ##################################################
    "This page explains the `AnimationEvent` class.": "このページでは`AnimationEvent`クラスについて説明します。",  # noqa
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "An animation-related event handler uses the `AnimationEvent` class, such as the complete animation event. Each animation interface passes this event instance to the handler.": "アニメーション終了時のイベントなどの各アニメーション関連のイベントで`AnimationEvent`クラスは使用されます。各アニメーションのインターフェイスはイベントのハンドラへこのイベントのインスタンスを渡します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the animation complete event handler. The animation interface passes the `AnimationEvent` instance argument as the `e: ap.AnimationEvent`:": "以下の例ではアニメーション完了時のイベントのハンドラへ`e: ap.AnimationEvent`という指定で`AnimationEvent`のインスタンスの引数を設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation is completed!")\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation is completed!")\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```',  # noqa
    ##################################################
    "## this property": "## this属性",
    ##################################################
    "The `AnimationEvent` instance's `this` property is a subclass instance of the `AnimationBase` class, such as the `AnimationMove`\\, `AnimationX`\\, or other class.": "`AnimationEvent`のインスタンスの`this`属性は`AnimationMove`や`AnimationX`等の`AnimationEvent`クラスのサブクラスになります。",  # noqa
    ##################################################
    "This type depends on the called interface, e.g., if you use the `animation_x` interface, `this` property type becomes an `AnimationX` instance.": "この属性の型は呼んだアニメーションのインターフェイスによって変動します。例えば`animation_x`のインターフェイスであれば`this`属性の型は`AnimationX`クラスのインスタンスとなります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    assert isinstance(e.this, ap.AnimationX)\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    assert isinstance(e.this, ap.AnimationX)\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```',  # noqa
    ##################################################
    "## Generic type annotation": "## ジェネリックの型アノテーション",
    ##################################################
    "The `AnimationEvent` class can set a generic type annotation. If you set a generic type annotation, then the animation target property type (e.g., `DisplayObject`) changes.": "`AnimationEvent`クラスはジェネリックの型アノテーションを行うことができます。もし型アノテーションをした場合にはアニメーション対象の値となる`target`属性は（`DisplayObject`などの）型アノテーションを行った型のインスタンスになります。",  # noqa
    ##################################################
    "The following example sets a `Rectangle` generic type annotation to the `AnimationEvent` and benefits from type checking libraries (such as the `mypy` or `Pylance`).": "以下のコードでは`AnimationEvent`の値に`Rectangle`のジェネリックの型アノテーションを行っており、mypyやPylanceなどでの型チェックのライブラリやエディタなどでの恩恵を受けることができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_x(x=50).start()\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_x(x=50).start()\n\n\nap.Stage(\n    stage_width=200,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#00aaff"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```',  # noqa
    ##################################################
    "## AnimationEvent constructor API": "## AnimationEvent クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Animation event class.<hr>": "アニメーションイベント用のクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `this`: AnimationBase": "- `this`: AnimationBase",
    ##################################################
    "  - Animation setting instance.": "  - アニメーションの設定を扱うインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).animation_complete(on_animation_complete)\n```': '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).animation_complete(on_animation_complete)\n```',  # noqa
    ##################################################
    "## this property API": "## this 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an animation setting instance of listening to this event.<hr>": "このイベントのリスナーとしてのアニメーション設定のインスタンスを取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `this`: AnimationBase": "- `this`: AnimationBase",
    ##################################################
    "  - Instance of listening to this event.": "  - このイベントのハンドラが設定されているインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).animation_complete(on_animation_complete)\n```': '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(x=100).animation_complete(on_animation_complete)\n```',  # noqa
}



"""The class implementation for the `thumb_down` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialThumbDownIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `thumb_down` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M15 3H6c-.83 0-1.54.5-1.84 1.22l-3.02 7.05c-.09.23-.14.47-.14.73v2c0 1.1.9 2 2 2h6.31l-.95 4.57-.03.32c0 .41.17.79.44 1.06L9.83 23l6.59-6.59c.36-.36.58-.86.58-1.41V5c0-1.1-.9-2-2-2zm4 0v12h4V3h-4z"/></svg>'  # noqa



"""Implementations of Polygon class.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._display import graphics
from apysc._display.add_to_parent_mixin import AddToParentMixIn
from apysc._display.append_fill_alpha_attr_expression_mixin import (
    AppendFillAlphaAttrExpressionMixIn,
)
from apysc._display.append_fill_color_expression_mixin import (
    AppendFillColorAttrExpressionMixIn,
)
from apysc._display.append_line_alpha_attr_expression_mixin import (
    AppendLineAlphaAttrExpressionMixIn,
)
from apysc._display.append_line_cap_attr_expression_mixin import (
    AppendLineCapAttrExpressionMixIn,
)
from apysc._display.append_line_color_attr_expression_mixin import (
    AppendLineColorAttrExpressionMixIn,
)
from apysc._display.append_line_joints_attr_expression_mixin import (
    AppendLineJointsAttrExpressionMixIn,
)
from apysc._display.append_line_point_mixin import AppendLinePointMixIn
from apysc._display.append_line_thickness_attr_expression_mixin import (
    AppendLineThicknessAttrExpressionMixIn,
)
from apysc._display.append_x_attr_expression_mixin import AppendXAttrExpressionMixIn
from apysc._display.append_y_attr_expression_mixin import AppendYAttrExpressionMixIn
from apysc._display.child_mixin import ChildMixIn
from apysc._display.css_mixin import CssMixIn
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.flip_x_mixin import FlipXMixIn
from apysc._display.flip_y_mixin import FlipYMixIn
from apysc._display.get_bounds_mixin import GetBoundsMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_caps import LineCaps
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_dot_setting_mixin import LineDashDotSettingMixIn
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dash_setting_mixin import LineDashSettingMixIn
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_dot_setting_mixin import LineDotSettingMixIn
from apysc._display.line_joints import LineJoints
from apysc._display.line_joints_mixin import LineJointsMixIn
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_round_dot_setting_mixin import LineRoundDotSettingMixIn
from apysc._display.polygon_append_constructor_expression_mixin import (
    PolygonAppendConstructorExpressionMixIn,
)
from apysc._display.rotation_around_center_mixin import RotationAroundCenterMixIn
from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
from apysc._display.scale_x_from_center_mixin import ScaleXFromCenterMixIn
from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
from apysc._display.scale_y_from_center_mixin import ScaleYFromCenterMixIn
from apysc._display.scale_y_from_point_mixin import ScaleYFromPointMixIn
from apysc._display.set_overflow_visible_setting_mixin import (
    SetOverflowVisibleSettingMixIn,
)
from apysc._display.set_x_and_y_with_minimum_point_interface_base import (
    SetXAndYWithMinimumPointInterfaceBase,
)
from apysc._display.skew_x_mixin import SkewXMixIn
from apysc._display.skew_y_mixin import SkewYMixIn
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom.point2d import Point2D
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.initialize_with_base_value_interface import (
    InitializeWithBaseValueInterface,
)
from apysc._mask.svg_mask_mixin import SvgMaskMixIn
from apysc._type.array import Array
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.repr_interface import ReprInterface
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Polygon(
    ReprInterface,
    XMixIn,
    AppendXAttrExpressionMixIn,
    YMixIn,
    AppendYAttrExpressionMixIn,
    SetOverflowVisibleSettingMixIn,
    CssMixIn,
    GraphicsBase,
    RotationAroundCenterMixIn,
    RotationAroundPointMixIn,
    ScaleXFromCenterMixIn,
    ScaleYFromCenterMixIn,
    ScaleXFromPointMixIn,
    ScaleYFromPointMixIn,
    FlipXMixIn,
    FlipYMixIn,
    SkewXMixIn,
    SkewYMixIn,
    AppendLinePointMixIn,
    SetXAndYWithMinimumPointInterfaceBase,
    FillColorMixIn,
    AppendFillColorAttrExpressionMixIn,
    FillAlphaMixIn,
    AppendFillAlphaAttrExpressionMixIn,
    LineColorMixIn,
    AppendLineColorAttrExpressionMixIn,
    LineAlphaMixIn,
    AppendLineAlphaAttrExpressionMixIn,
    AppendLineThicknessAttrExpressionMixIn,
    AppendLineCapAttrExpressionMixIn,
    LineJointsMixIn,
    AppendLineJointsAttrExpressionMixIn,
    LineDotSettingMixIn,
    LineDashSettingMixIn,
    LineRoundDotSettingMixIn,
    LineDashDotSettingMixIn,
    PolygonAppendConstructorExpressionMixIn,
    GetBoundsMixIn,
    VariableNameSuffixMixIn,
    InitializeWithBaseValueInterface,
    AddToParentMixIn,
    SvgMaskMixIn,
):
    """
    The polygon vector graphics class.

    References
    ----------
    - Polygon class
        - https://simon-ritchie.github.io/apysc/en/polygon.html
    - Graphics draw_polygon interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_polygon.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
    ...     points=[
    ...         ap.Point2D(x=50, y=50),
    ...         ap.Point2D(x=50, y=100),
    ...         ap.Point2D(x=100, y=75),
    ...     ]
    ... )
    >>> polygon.fill_color
    Color("#00aaff")
    """

    # self
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    # points
    @arg_validation_decos.are_point_2ds(arg_position_index=1)
    # fill_color
    @arg_validation_decos.is_color(arg_position_index=2, optional=False)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=3, optional=False)
    # line_color
    @arg_validation_decos.is_color(arg_position_index=4, optional=False)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=5, optional=False)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=6, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6, optional=False)
    # line_cap
    @arg_validation_decos.is_line_cap(arg_position_index=7, optional=True)
    # line_joints
    @arg_validation_decos.are_line_joints(arg_position_index=8, optional=True)
    # line_dot_setting
    @arg_validation_decos.is_line_dot_setting(arg_position_index=9)
    # line_dash_setting
    @arg_validation_decos.is_line_dash_setting(arg_position_index=10)
    # line_round_dot_setting
    @arg_validation_decos.is_line_round_dot_setting(arg_position_index=11)
    # line_dash_dot_setting
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=12)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=13, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=14, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        points: Union[List[Point2D], Array[Point2D]],
        fill_color: Color = COLORLESS,
        fill_alpha: Union[float, Number] = 1.0,
        line_color: Color = COLORLESS,
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        line_cap: Optional[Union[String, LineCaps]] = None,
        line_joints: Optional[Union[String, LineJoints]] = None,
        line_dot_setting: Optional[LineDotSetting] = None,
        line_dash_setting: Optional[LineDashSetting] = None,
        line_round_dot_setting: Optional[LineRoundDotSetting] = None,
        line_dash_dot_setting: Optional[LineDashDotSetting] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a polygon vector graphic. This class is
        similar to the Polyline class, but unlike that,
        this class connects an end-point and start-point.

        Parameters
        ----------
        points : Array of Point2D or list of Point2D
            List of polygon vertex points.
        fill_color : Color, default COLORLESS
            A fill-color to set.
        fill_alpha : float or Number, default 1.0
            A fill-alpha to set.
        line_color : Color, default COLORLESS
            A line-color to set.
        line_alpha : float or Number, default 1.0
            A line-alpha to set.
        line_thickness : int or Int, default 1
            A line-thickness (line-width) to set.
        line_cap : String or LineCaps or None, default None
            A line-cap setting to set.
        line_joints : String or LineJoints or None, default None
            A line-joints setting to set.
        line_dot_setting : LineDotSetting or None, default None
            A dot setting to set.
        line_dash_setting : LineDashSetting or None, default None
            A dash setting to set.
        line_round_dot_setting : LineRoundDotSetting or None, default None
            A round-dot setting to set.
        line_dash_dot_setting : LineDashDotSetting or None, default None
            A dash-dot (1-dot chain) setting to set.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If the specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - Polygon class
            - https://simon-ritchie.github.io/apysc/en/polygon.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> polygon: ap.Polygon = ap.Polygon(
        ...     points=[
        ...         ap.Point2D(x=50, y=50),
        ...         ap.Point2D(x=50, y=100),
        ...         ap.Point2D(x=100, y=75),
        ...     ],
        ...     fill_color=ap.Color("#00aaff"),
        ... )
        >>> polygon.fill_color
        Color("#00aaff")
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        if isinstance(points, list):
            points = Array(points)
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.POLYGON
        )
        self.variable_name = variable_name
        self._set_initial_basic_values(
            fill_color=fill_color,
            fill_alpha=fill_alpha,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=line_cap,
            line_joints=line_joints,
        )
        self.points = points
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            line_dot_setting=line_dot_setting,
            line_dash_setting=line_dash_setting,
            line_round_dot_setting=line_round_dot_setting,
            line_dash_dot_setting=line_dash_dot_setting,
        )
        self._set_x_and_y_with_minimum_point()
        super(Polygon, self).__init__(variable_name=variable_name)
        self._set_overflow_visible_setting()
        self._add_to_parent(parent=parent)

    @final
    def _set_x_and_y_with_minimum_point(self) -> None:
        """
        Set an x and y properties coordinate with a minimum point.
        """
        min_x: float = min([point._x._value for point in self._points._value])
        min_y: float = min([point._y._value for point in self._points._value])

        suffix: str = self._get_attr_or_variable_name_suffix(value_identifier="x")
        self._x = Number(min_x, variable_name_suffix=suffix)

        suffix = self._get_attr_or_variable_name_suffix(value_identifier="y")
        self._y = Number(min_y, variable_name_suffix=suffix)

    @classmethod
    @final
    def _create_with_graphics(
        cls,
        *,
        graphics: "graphics.Graphics",
        points: Union[List[Point2D], Array[Point2D]],
        variable_name_suffix: str = "",
    ) -> "Polygon":
        """
        Create a polygon instance with the instance of
        specified graphics.

        Parameters
        ----------
        graphics : Graphics
            Graphics instance to link this instance.
        points : Array[Point2D]
            List of polygon vertex points.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        polygon : Polygon
            A created polygon instance.
        """
        polygon: Polygon = Polygon(
            points=points,
            fill_color=graphics._fill_color,
            fill_alpha=graphics._fill_alpha,
            line_color=graphics._line_color,
            line_alpha=graphics._line_alpha,
            line_thickness=graphics._line_thickness,
            line_cap=graphics._line_cap,
            line_joints=graphics._line_joints,
            line_dot_setting=graphics._line_dot_setting,
            line_dash_setting=graphics._line_dash_setting,
            line_round_dot_setting=graphics._line_round_dot_setting,
            line_dash_dot_setting=graphics._line_dash_dot_setting,
            parent=graphics,
            variable_name_suffix=variable_name_suffix,
        )
        return polygon

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Polygon("<variable_name>")`).
        """
        repr_str: str = f'{Polygon.__name__}("{self.variable_name}")'
        return repr_str

    @classmethod
    @final
    def _initialize_with_base_value(cls) -> "Polygon":
        """
        Initialize this class with a base value(s).

        Returns
        -------
        polygon : Polygon
            An initialized polygon instance.
        """
        from apysc._type.boolean import Boolean

        polygon: Polygon = Polygon(
            points=[
                Point2D(x=-2, y=-2),
                Point2D(x=-1, y=-2),
                Point2D(x=-1, y=-1),
            ]
        )
        polygon.visible = Boolean(False)
        return polygon
